Eventemitter = require('events').EventEmitter
dchannelsLib = new Eventemitter()
child_process = require 'child_process'
_ = require 'lodash'
isdnLib = require "./isdn"

helpers = require './channels-helpers'

debug = require("debug")("omg-channels:dchannels")

running = false

isdnStates = []
isdnLib.on "new", (states) ->
  isdnStates = states
  dchannelsLib.emit "changed"

dchannelsList = []
getDchannels = ->
  command = "getchannels -m -d 2>&1 | sort -V | grep -v 'Channels found'"
  child_process.exec command, (error, stdout) ->
    channels = stdout.split("\n")
    channels = _.filter channels, (line) ->
      return line isnt ""

    channels = _.map channels, (chan) ->
      # Channel looks like this: 11.b/D/PCM/loop/0/0/nil
      # Split channel in parts.
      chanParts = chan.split "/"
      [ilidNumber, ilidIndex] = chanParts[0].split(".") # = ["11", "b"]
      dchannelObject =
        name: chanParts[0] # == 11.b
        ilidType: chanParts[2] # == PCM
        ilidTypeUser: helpers.getChannelType(chanParts[2]) # == PCM
        ilidNumber: parseInt(ilidNumber, 10) # == 11
        ilidIndex: ilidIndex # == "b"
        ilidIndexUser: helpers.convertLetterToNumber ilidIndex # == 2

      # set a temporary index which we can sort by
      dchannelObject.channelIndex = parseInt("#{dchannelObject.ilidNumber}#{dchannelObject.ilidIndexUser}", 10)
      return dchannelObject

    # sort channels by temporary index and set final index number and port
    channels = _.sortBy channels, "channelIndex"
    _.each channels, (chan, index) ->
      chan.channelIndex = index + 1
      chan.port = 3217 + index

    # set dchannelsList to new channels
    dchannelsList = channels
    debug "Got dchannelsList"
    dchannelsLib.emit "changed"
getDchannels()

dchannelConfigurations = []
getDchannelConfigurations = ->
  # dchannelConfigurations.removeAll()
  dchannelConfigurations = []
  confJson = ""
  fullConfigs = []
  confGetProc = child_process.spawn "omgconfiginfo", ["--json"]
  confGetProc.stdout.on "data", (data) ->
    confJson = "#{confJson}#{data}"
  confGetProc.on "close", ->
    try
      confJson = JSON.parse(confJson)
      fullConfigs = confJson.struct_Box.struct_DChannel
    catch e
      debug chalk.red("error parsing dchannel config"), e

    _.each fullConfigs, (conf) ->
      tempChan =
        qsigActive: (if conf.bool_Qsig and conf.short_FacilityMode isnt -1 then true else false)
        activated: (if conf.bool_CfgValid then true else false)
        # typeDisplayName: helpers.getChannelType(conf.string_IlidType)
        ILID: "#{conf.string_IlidSlot}.#{conf.string_IlidIndex}"
        ilidTitle: conf.string_IlidTitle
      # tempChan = conf
      # index: parseInt index
      # port: 3217 + tempChan.index
      # sortName: pad tempChan.index
      dchannelConfigurations.push tempChan
      dchannelsLib.emit "changed"
getDchannelConfigurations()

dchannels = []
dchannelsLib.on "changed", ->
  return unless dchannelConfigurations.length and dchannelsList.length and isdnStates.length
  #reset dchannels
  dchannels = []

  _.each dchannelsList, (channel, index) ->
    oneMergedDchannel = {}
    dchanIlid = "#{channel.ilidNumber}.#{channel.ilidIndex}"

    #find matching isdn status
    dchanIsdn = _.find isdnStates, (i) ->
      i.ILID is dchanIlid
    # mergeDchannel with matching isdn status
    _.merge channel, (dchanIsdn or {L1:"OFF", L2: "OFF", CALLS:0 })

    #find matching dchannel configuration
    dchanConf = _.find dchannelConfigurations, (dcc) ->
      dcc.ILID is dchanIlid
    #merge dchannnel with configuration
    _.merge channel, (dchanConf or {})

    amountOfChannelsOfSameType = _.countBy(dchannels, "ilidTypeUser")[channel.ilidTypeUser] or 0
    channel.name = channel.ilidTitle or "#{channel.ilidTypeUser} #{amountOfChannelsOfSameType + 1}" # "#{channel.ilidTypeUser} #{channel.ilidIndexUser}"

    dchannels.push channel

  dchannelsLib.emit "new", dchannelsList


dchannelsLib.getNewConfig = ->
  return unless running
  getDchannelConfigurations()

dchannelsLib.start = ->
  debug "start called"
  if running
    debug "already running, not starting again"
    return
  getDchannels()
  isdnLib.start()
  running = true
  debug "starting dchannels"
dchannelsLib.stop = ->
  debug "stop called"
  unless running
    debug "already stopped, not doing anything"
    return
  isdnLib.stop()
  running = false


module.exports = dchannelsLib