/* exported should */
/* exported co */
/* jshint expr: true */
'use strict';

require("coffee-script");

var config = require("/data/config.json");
var aggConfig = config.agg;
var zmq = require('zmq');
var host = aggConfig.protocol + "://" + aggConfig.address + ":";
var pullSocket = zmq.socket("pull");
pullSocket.bindSync("" + host + aggConfig.subscriberPoolPort);

var omg_tcpdump = require('../lib/omg-tcpdump');

var exec = require("child_process").exec;

describe('omg_tcpdump', function() {
    describe('omg_tcpdump.start', function() {
        it('should listen for 0.1 minute when handed 0.1 as a string as an argument', function(done) {
            this.timeout(1000 * 60 * 0.2 + 100);
            omg_tcpdump.start('eth0', '0.1');
            var onMessage = function(data) {
                data = JSON.parse(data);
                data.producer.should.be.exactly("tcpdump");
                console.log(data);
                if (data.status === "started") {
                    console.log("started successfuly");
                }
                if (data.data) {
                    console.log(data.data);
                }
                if (data.status === "stopped") {
                    done();
                    pullSocket.removeListener("message", onMessage);
                }
            };
            pullSocket.on("message", onMessage);
        });

        it('should send error if already running', function(done) {
            console.log("\n\n\n\n\n\n\n");
            var runTime = 0.1,
                timeout = runTime * 60 * 1000 + 1000;
            this.timeout(timeout);
            omg_tcpdump.start('eth0', runTime);
            setTimeout(function() {
                omg_tcpdump.start('eth0', runTime);
            }, 100);
            var onMessage = function(data) {
                data = JSON.parse(data);
                if(data && data.error && data.error === "tcpdump already running") {
                    omg_tcpdump.stop();
                }
                if (data.status === "stopped") {
                    done();
                    pullSocket.removeListener("message", onMessage);
                }
            };
            pullSocket.on("message", onMessage);
        });

        it('should stop dump when asked for', function(done) {
            console.log("\n\n\n\n\n\n\n");
            var stopafter = 0.5;
            this.timeout(1000 * 60 * stopafter + 1000);
            omg_tcpdump.start('eth0', '4.1');
            var onMessage = function(data) {
                data = JSON.parse(data);
                data.producer.should.be.exactly("tcpdump");
                console.log(data);
                if (data.status === "started") {
                    console.log("started successfuly");
                }
                if (data.data) {
                    console.log(data.data);
                }
                if (data.status === "stopped") {
                    done();
                    pullSocket.removeListener("message", onMessage);
                }
            };

            setTimeout(function() {
                omg_tcpdump.stop();
            }, 1000 * 60 * stopafter);
            pullSocket.on("message", onMessage);
        });

        it('should send "stopped" when exiting unexpected', function(done) {
            console.log("\n\n\n\n\n\n\n");
            var killTime = 10 * 1000,
                normalRuntime = 60 * 1000,
                timeout = normalRuntime + 1000;

            this.timeout(timeout);

            omg_tcpdump.start('eth0', '4.1');

            var onMessage = function(data) {
                data = JSON.parse(data);
                data.producer.should.be.exactly("tcpdump");
                console.log(data);
                if (data.status === "started") {
                    console.log("started successfuly");
                }
                if (data.data) {
                    console.log(data.data);
                }
                if (data.status === "stopped") {
                    done();
                    pullSocket.removeListener("message", onMessage);
                }
            };

            setTimeout(function() {
                exec("killall tcpdump");
            }, killTime);
            pullSocket.on("message", onMessage);
        });

        /* jshint ignore:start */

        return;

        it('should listen for 4.1 minute when handed 4.1 as a string as an argument', function(done) {
            this.timeout(1000 * 60 * 4.1 + 1000);
            omg_tcpdump.start('eth0', '4.1');
            var onMessage = function(data) {
                data = JSON.parse(data);
                data.producer.should.be.exactly("tcpdump");
                console.log(data);
                if (data.status === "started") {
                    console.log("started successfuly");
                }
                if (data.data) {
                    console.log(data.data);
                }
                if (data.status === "stopped") {
                    done();
                    pullSocket.removeListener("message", onMessage);
                }
            };
            pullSocket.on("message", onMessage);
        });

        it('should listen for 10.1 minute when handed 10.1 as a string as an argument', function(done) {
            this.timeout(1000 * 60 * 10.1 + 1000);
            omg_tcpdump.start('eth0', '10.1');
            var onMessage = function(data) {
                data = JSON.parse(data);
                data.producer.should.be.exactly("tcpdump");
                console.log(data);
                if (data.status === "started") {
                    console.log("started successfuly");
                }
                if (data.data) {
                    console.log(data.data);
                }
                if (data.status === "stopped") {
                    done();
                    pullSocket.removeListener("message", onMessage);
                }
            };
            pullSocket.on("message", onMessage);
        });


        it('should listen for 20.1 minute when handed 20.1 as a string as an argument', function(done) {
            this.timeout(1000 * 60 * 20.1 + 1000);
            omg_tcpdump.start('eth0', '20.1');
            var onMessage = function(data) {
                data = JSON.parse(data);
                data.producer.should.be.exactly("tcpdump");
                console.log(data);
                if (data.status === "started") {
                    console.log("started successfuly");
                }
                if (data.data) {
                    console.log(data.data);
                }
                if (data.status === "stopped") {
                    done();
                    pullSocket.removeListener("message", onMessage);
                }
            };
            pullSocket.on("message", onMessage);
        });


        it('should listen for 30.1 minute when handed 30.1 as a string as an argument', function(done) {
            this.timeout(1000 * 60 * 30.1 + 1000);
            omg_tcpdump.start('eth0', '30.1');
            var onMessage = function(data) {
                data = JSON.parse(data);
                data.producer.should.be.exactly("tcpdump");
                console.log(data);
                if (data.status === "started") {
                    console.log("started successfuly");
                }
                if (data.data) {
                    console.log(data.data);
                }
                if (data.status === "stopped") {
                    done();
                    pullSocket.removeListener("message", onMessage);
                }
            };
            pullSocket.on("message", onMessage);
        });

        /* jshint ignore:end */

    });

    // describe('omg_tcpdump.stop', function() {
    //     console.log("5");
    //     it('should stop the currently running tcpdump', function(done) {
    //         omg_tcpdump.stop(function(err, data) {
    //             if (err) {
    //                 throw err;
    //             }
    //             console.log(data);
    //             done();
    //         });
    //     });
    // });


});