child_process = require 'child_process'
debug = require("debug")("omg-testcall.js:Call")
ko = require 'knockout'

events = require 'events'
EventEmitter = events.EventEmitter

testcallFolder = "/tmp/testcall"

appConfig = global.appConfig
runAsRoot = appConfig.sudo or false if appConfig

class Call extends EventEmitter
  constructor: ->
    @running = ko.observable(false)

  start: (callObject) ->
    if @running()
      debug "Call still running, wait!"
      return

    debug "Starting call"
    @running true

    #get some call settings and set path to coresponding options.xml
    redirectedNumber = callObject.redirectedNumber
    callingPartyName = callObject.callerName
    optionsPath = "#{testcallFolder}/options.xml"
    if redirectedNumber and not callingPartyName
      optionsPath = "#{testcallFolder}/options_redirected.xml"
    if not redirectedNumber and callingPartyName
      optionsPath = "#{testcallFolder}/options_callername.xml"
    if redirectedNumber and callingPartyName
      optionsPath = "#{testcallFolder}/options_callername_redirected.xml"

    #create options for sipp
    sippOptions = ["-skip_rlimit -silent -max_socket 50000 -sf #{optionsPath} -mp 6000 -trace_msg -trace_shortmsg -trace_logs -trace_err -m 1 -users 1 -pause_msg_ign"]

    child_process.exec "#{__dirname}/bash/primaryAddress.sh", (error, stdout) => #, stderr
      if error
        msg = "Could not dertermine localAddress! (#{__dirname}/bash/primaryAddress.sh failed!)"
        @emit "error", error
        debug "error getting primaryAddress: #{error}"
        return

      localAddress = stdout.replace(/\n/g, "")

      ## setting up command with arguments
      if runAsRoot
        command = "sudo"
        args = ["sipp"]
      else
        command = "sipp"
        args = []

      args.push "#{callObject.host}:#{callObject.port}" #127.0.0.1:5060
      switch callObject.proto.toUpperCase()
        when "TCP" then args.push "-t t1"
        when "TLS" then args.push "-t l1"
        when "UDP" then args.push "-t u1"

      args.push "-i #{localAddress}" # -i 127.0.0.1 (used in SIP contact: IA: FROM:)
      args.push "-mi #{localAddress}" # -i 127.0.0.1 (media port)
      args.push "-s #{callObject.calledPartyNumber}"
      args.push "-set callingPartyNumber #{callObject.callingPartyNumber}"
      args.push "-set redirectedNumber #{redirectedNumber}" if redirectedNumber
      args.push "-set callingPartyName #{callingPartyName}" if callingPartyName

      argCmd = args.concat(sippOptions)
      argCmd = argCmd.join(" ").split(new RegExp(" "))

      callProcess = child_process.spawn command, argCmd, {cwd: "/tmp/"}
      @emit "status", "started"
      @emit "status", "running"

      callProcess.on "error", (err) =>
        @emit "status", "stopped"
        @emit "error", err
        return

      #running calldata get's returned through stderr who knows why? Gido?
      callProcess.stderr.on "data", (data) =>

        data = data.toString("utf-8")
        # check if a testcall is already in progress.
        match = data.match("Address already in use")

        debug "callProcess error: #{data}"

        normalStdErr = data.match "Generated SSRC"

        @emit "data", [data] unless normalStdErr

      callProcess.on "close", (code) =>
        @emit "finished"
        @running false

module.exports = new Call()