#!/usr/bin/env coffee

freeport = require 'freeport'
fs = require 'fs'
zmq = require 'zmq'
debug = require("debug")("omg-porthandler")
completeConfig = require "omg-configloader"
config = completeConfig.zmq
porthandlerConfig = config.porthandler

transport = porthandlerConfig.transport or config.standardTransport
address = porthandlerConfig.address or config.standardAddress
port = porthandlerConfig.rep
replyPort = zmq.socket "rep"
replyPort.bindSync "#{transport}://#{address}:#{port}"


usedPorts = []
componentConfigurations = {}

sendError = (eText) ->
  errorResponse =
    error: eText
  replyPort.send JSON.stringify(errorResponse)

sendPort = (port) ->
  replyPort.send port

getFreePort = (cb) ->
  freeport (err, port) ->
    if err or usedPorts.indexOf(port) > -1
      getFreePort cb
      return
    else
      cb port


replyPort.on "message", (componentName, portType) ->
  messageParts = Array.apply(null, arguments)
  unless componentName and portType
    sendError "Specify 'componentName' and 'portType'!"
    return

  # componentName = componentName.toString()
  # portType = portType.toString()

  combinedName = "#{componentName}#{portType}"

  if componentConfigurations[combinedName]
    debug combinedName, "was requested before:", componentConfigurations[combinedName]
    sendPort componentConfigurations[combinedName]
  else
    debug combinedName, "was not yet requested. Getting a new free port"
    getFreePort (port) ->
      componentConfigurations[combinedName] = port
      usedPorts.push port
      debug "found free port:", port
      sendPort port
      # console.log "sending new port #{port}"
      fs.writeFile "#{__dirname}/portConfig.json", JSON.stringify(componentConfigurations, undefined, 4)
